//
//      AWSProduct.cpp
//
//      This source code was automatically
//      generated by SophiaFramework 4.1.1
//

#include "pself3.hpp"
#include "AWSTabControl.hpp"
#include "AWSProduct.hpp"

//++++++++++++++++++++++++++AWSProduct class implementation+++++++++++++++++++++++++++++//

// Constructor
AWSProduct::AWSProduct() :
	_imageUrl(null), _offerSummary(null), _offers(null), _customerReviews(null), 
	_similarProducts(null), _browserNodes(null), _listmaniaLists(null)
{
    return;
}

// Destructor
AWSProduct::~AWSProduct(Void)
{
	ClearData();
}

// Factory function
AWSProductPtr AWSProduct::Factory(SFXAnsiString producttype) 
{
	if(producttype == BOOK_SEARCH) 
		return	::new AWSBook();
	
	if(producttype == DVD_SEARCH) 
		return	::new AWSDvd();
	
	return null;
}

//get url for itemlookup
SFXAnsiString AWSProduct::GetUrlForItemLookup(SFXAnsiStringConstRef asin)
{
    return pself3::GetInstance()->GetLocale() + AMAZON_URL_ITEMLOOKUP_PREFIX + asin + AMAZON_URL_POSTFIX;
}

//get url for keyword search
SFXAnsiString AWSProduct::GetUrlForKeyWordSearch(SFXAnsiStringConstRef searchindex, SFXAnsiStringConstRef browsenode, SFXAnsiStringConstRef keyword)
{
	SFXAnsiString url;
	

	url = pself3::GetInstance()->GetLocale() + AMAZON_URL_PREFIX+ searchindex;
	
	if (browsenode !="All")
		url = url + AMAZON_URL_BROWSENODE + browsenode;

	if(!keyword.IsEmptyCString())
		url = url + AMAZON_URL_KEYWORD + keyword;

	url = url + AMAZON_URL_POSTFIX;

    return url;
}

// release resource
Void AWSProduct::ClearData(Void)
{
	_title.Clear();
	_asin.Clear();

	delete _imageUrl;
	_imageUrl = null;
    
	delete _offerSummary;
	_offerSummary = null;
  
	if (_offers != null) 
	{
		SFXArray<OfferRecPtr>::Enumerator enumerator = _offers->offers.GetFirstEnumerator();
		while (enumerator.HasNext()) 
		{
			OfferRecPtr offer(enumerator.GetNext());
			delete offer;
		}
		_offers->offers.Clear();
		delete _offers;
		_offers = null;
	}
    
	if (_customerReviews != null) 
	{
		SFXArray<ReviewRecPtr>::Enumerator enumerator = _customerReviews->reviews.GetFirstEnumerator();
		while (enumerator.HasNext()) 
		{
			ReviewRecPtr review(enumerator.GetNext());
			delete review;
		}
		_customerReviews->reviews.Clear();
		delete _customerReviews;
		_customerReviews = null;
	}

	if (_similarProducts != null) 
	{
		SFXArray<SimilarProductRecPtr>::Enumerator enumerator = _similarProducts->similarProducts.GetFirstEnumerator();
		while (enumerator.HasNext()) 
		{
			SimilarProductRecPtr similarProduct(enumerator.GetNext());
			delete similarProduct;
		}
		_similarProducts->similarProducts.Clear();
		delete _similarProducts;
		_similarProducts = null;
    }
}

// helper function
SFXAnsiString AWSProduct::DecodeHtml(SFXAnsiStringConstRef string)
{
    return string.Replace("&lt;", '<').Replace("&gt;", '>').Replace("&quot;", '"').Replace("&amp;", '&');
}

// helper function
SFXDate AWSProduct::StringToDate(SFXAnsiStringConstRef string)
{
    SFXDate ret(0);

    // YYYY/MM/DD ` YYYY/MM `gĂ悤łB
    // YYYY/MM `̎̃f[^͖̈B
    if (string.GetLength() >= 10) {
        ret.SetDate(static_cast<SInt16>(string.Substring(0, 4).AsUInt32()),
                    static_cast<SInt16>(string.Substring(5, 7).AsUInt32()),
                    static_cast<SInt16>(string.Substring(8, 10).AsUInt32()));
    }
    else if (string.GetLength() >= 7) {
        ret.SetDate(static_cast<SInt16>(string.Substring(0, 4).AsUInt32()),
                    static_cast<SInt16>(string.Substring(5, 7).AsUInt32()), 1);
    }
    return ret;
}


//this helper function is used by MakeReviewPane
SInt32 SearchBrTag(SFXAnsiStringConstRef string, SInt32 index = -1)
{
    SInt32 brc(string.FirstIndexOf("<BR />", index));
    SInt32 br(string.FirstIndexOf("<br />", index));
    SInt32 pc(string.FirstIndexOf("<P>", index));
    SInt32 p(string.FirstIndexOf("<p>", index));

    br = (brc < br && brc >= 0 || br < 0) ? brc : br;
    pc = (pc < p && pc >= 0 || p < 0) ? pc : p;
    return (br < pc && br >= 0 || pc < 0) ? br : pc;
}

//This pane is identical to all products
SFCError AWSProduct::MakeReviewPane(SFRTabControlPtr tab)
{
	SFCError error(SFERR_NO_ERROR);
	ScrollTabPanePtr pane;

    SInt16 i, height;

	if ((pane = ::new ScrollTabPane(tab, "Reviews")) == null) 
		return SFERR_NO_MEMORY;
   
    if ((error = pane->static_catch()) != SFERR_NO_ERROR) 
		return error;

	SFXRectangle temp(pane->GetContentWorld().Deflate(CONTROL_MARGIN, CONTROL_MARGIN));
	temp.SetHeight(SFXGraphics::GetFontHeight(AEE_FONT_NORMAL));

    if (_customerReviews == null) 
	{
		if ((::new SFRLabelControl(pane, temp, "No review"), SFRLabelControl::ALIGN_LEFT) == null) 
			return SFERR_NO_MEMORY;

		return SFERR_NO_ERROR;
	}

	if ((::new SFRLabelControl(pane, temp, SFXWideString::Format("Average Rating: %d", _customerReviews->averageRating), SFRLabelControl::ALIGN_LEFT)) == null)
		return SFERR_NO_MEMORY;

	temp.AddY(temp.GetHeight());
	
	for (i = 0; i < _customerReviews->reviews.GetSize() && i < 5; ++i) 
	{ // display up to 5 reviews
		temp.AddY(3 * CONTROL_MARGIN);
		temp.SetHeight(SFXGraphics::GetFontHeight(AEE_FONT_NORMAL));
		SFRLabelControlPtr label(::new SFRLabelControl(pane, temp, SFXWideString::Format("Reviewer%d: %d", i + 1, _customerReviews->reviews[i]->rating), SFRLabelControl::ALIGN_LEFT));
                
		if (label == null) 
			return SFERR_NO_MEMORY;
		
		label->SetColor(SFRLabelControl::COLOR_TITLE, SFXRGBColor(0xFF, 0x00, 0x00, 0x00));
		temp.AddY(temp.GetHeight() + CONTROL_MARGIN);
		height = SFXGraphics::MeasureString(AEE_FONT_NORMAL, _customerReviews->reviews[i]->summary, temp.GetWidth());
		temp.SetHeight(height);

		label = ::new SFRLabelControl(pane, temp, _customerReviews->reviews[i]->summary, SFRLabelControl::ALIGN_LEFT);
                    
		if (label == null) 
			return SFERR_NO_MEMORY;

		label->SetColor(SFRLabelControl::COLOR_TITLE, SFXRGBColor(0x00, 0x00, 0xFF, 0x00));
		temp.AddY(temp.GetHeight() + CONTROL_MARGIN);

		SFXAnsiStringConstRef string(_customerReviews->reviews[i]->content);

		SInt32 br1(SearchBrTag(string));
			
		br1 = (br1 > -1) ? br1 : SINT32_MAXIMUM;
		SInt32 br2(0);

		do {
			SFXWideString tempString(string.Substring(br2, br1));
			height = SFXGraphics::MeasureString(AEE_FONT_NORMAL, tempString, temp.GetWidth());
			temp.SetHeight(height);
				
			if ((::new SFRLabelControl(pane, temp, tempString, SFRLabelControl::ALIGN_LEFT)) == null) 
				return SFERR_NO_MEMORY;

			temp.AddY(temp.GetHeight());
			
			// br1: BR or P?
			br2 = string.FirstIndexOf('>', br1) + 1;

			br1 = SearchBrTag(string, br2);
			if (br1 < 0)	br1 = SINT32_MAXIMUM;
		} while (br2 > 0);
	}//end for
       
	pane->SetHeight(temp.GetTop());

	return error;
}

//This pane is identical to all products
SFCError AWSProduct::MakeSimilarPane(SFRTabControlPtr tab)
{
    SFCError error(SFERR_NO_ERROR);
    AWSTabPanePtr pane;
    UInt16 i;

	if (_similarProducts == null) 
		return SFERR_NO_ERROR;

	if ((pane = ::new AWSTabPane(tab, "Related Products")) == null) return SFERR_NO_MEMORY;

    if ((error = pane->static_catch()) != SFERR_NO_ERROR) 
		return error;

	for (i = 0; i < _similarProducts->similarProducts.GetSize(); ++i) 
	{
		if ((error = pane->Append("ASIN", _similarProducts->similarProducts[i]->asin)) != SFERR_NO_ERROR)
			return error;
		
		if ((error = pane->Append("Title", _similarProducts->similarProducts[i]->title)) != SFERR_NO_ERROR)
			return error;
	}

    error = pane->MakeLabels();
    return error;
}

//This pane is identical to all products
SFCError AWSProduct::MakeImagePane(SFRTabControlPtr tab)
{
	SFCError error(SFERR_NO_ERROR);

	if (_imageUrl == null) return SFERR_FAILED;

	if ((::new AWSImageTabPane(tab, "Image", _imageUrl->mediumImageURL, _asin))==null)
        return SFERR_NO_MEMORY;

	return error;
}

// dispaly product information
SFCError AWSProduct::DisplayMe(SFXAnsiStringConstRef producttype, bool addedFavorite)
{
	AWSDisplayWindowPtr _displaywindow;
	SFRTabControlPtr _tab;
    SFXRectangle _rect;
    SFCError error(SFERR_NO_ERROR);

	//create display window
	if ((_displaywindow  = ::new AWSDisplayWindow(producttype, _asin, _title, addedFavorite)) == null)
		return SFERR_NO_MEMORY;

    _rect = _displaywindow->GetContentWorld();
    _rect.AddTop(TOP_MARGIN);

	//create tab control
    if ((_tab = ::new SFRTabControl(_displaywindow, _rect)) == null)
       return SFERR_NO_MEMORY;
	//
	//making panes
	//
	if ((error = _tab->static_catch()) != SFERR_NO_ERROR)
		return error;

	// make general pane
	if ((error = MakeGeneralPane(_tab)) != SFERR_NO_ERROR)
		return error;
	// make detail pane
	if ((error = MakeDetailPane(_tab)) != SFERR_NO_ERROR)
		return error;
	// make review pane
	if ((error = MakeReviewPane(_tab)) != SFERR_NO_ERROR)
	    return error;
	// make similar pane
    if ((error = MakeSimilarPane(_tab)) != SFERR_NO_ERROR)
		return error;
    // make image pane 
    if ((error = MakeImagePane(_tab)) != SFERR_NO_ERROR)
		return error;

	return error;
}
//++++++++++++++++++++++End of AWSProduct class implementation++++++++++++++++++++++++++//


//++++++++++++++++++++++++++AWSBook class implementation+++++++++++++++++++++++++++++//
AWSBook::AWSBook(Void): AWSProduct()
{
	_itemAttributes = null;
	return;
}

AWSBook::~AWSBook(Void)
{
	if (_itemAttributes != null)
	{
		delete _itemAttributes;
		_itemAttributes = null;
	}

	return;
}


// Set ItemAtributes
Void	AWSBook::SetItemAttributes(SFXAnsiString param)
{
	if (_flag == "Author")
	{
		SFXAnsiStringPtr author = new SFXAnsiString(DecodeHtml(param).GetCString());
		_itemAttributes->authors.Append(author);
	}
	else if (_flag == "Binding")
		_itemAttributes->binding = DecodeHtml(param);
	else if (_flag == "Amount")
		_itemAttributes->amount = param.AsSInt32();
	else if (_flag == "CurrencyCode")
		_itemAttributes->currencyCode = DecodeHtml(param);
	else if (_flag == "FormattedPrice")
		_itemAttributes->formattedPrice = DecodeHtml(param);
	else if (_flag == "NumberOfPages")
		_itemAttributes->numberOfPages = param.AsSInt32();
	else if (_flag == "Length")
		_itemAttributes->packageDimensionsLength = param.AsSInt32();
	else if (_flag == "Width")
		_itemAttributes->packageDimensionsWidth = param.AsSInt32();
	else if (_flag == "PublicationDate")
		_itemAttributes->publicationDate = StringToDate(param);
	else if (_flag == "Publisher")
		_itemAttributes->publisher = DecodeHtml(param);
	else if (_flag == "Title")
		SetTitle(param);
	else return;
}

// Create pane to display general information
SFCError AWSBook::MakeGeneralPane(SFRTabControlPtr tab)
{
    SFCError error(SFERR_NO_ERROR);
    AWSTabPanePtr pane;
    UInt16 i;
    
	if (_itemAttributes == null) return SFERR_FAILED;

	if ((pane = ::new AWSTabPane(tab, "General")) == null) return SFERR_NO_MEMORY;

	if ((error = pane->static_catch()) != SFERR_NO_ERROR)
		return error;
    
	if ((error = pane->Append("Title", _title)) != SFERR_NO_ERROR)
		return error;
	
	if (_itemAttributes->authors.GetSize()>0)
	{
		SFXAnsiString authors = _itemAttributes->authors[0]->TrimLeft();
		for (i = 1; i < _itemAttributes->authors.GetSize(); ++i) 
			authors = authors + ", " + _itemAttributes->authors[i]->TrimLeft();
		if ((error = pane->Append("Author(s)", authors)) != SFERR_NO_ERROR)
			return error;
	}
	if ((error = pane->Append("Price", _itemAttributes->formattedPrice)) != SFERR_NO_ERROR)
		return error;
	 
	error = pane->MakeLabels();
    return error;
}

// create pane for detailed information
SFCError AWSBook::MakeDetailPane(SFRTabControlPtr tab)
{
    SFCError error(SFERR_NO_ERROR);
    AWSTabPanePtr pane;

	if (_itemAttributes == null) return SFERR_FAILED;

	if ((pane = ::new AWSTabPane(tab, "Details")) == null) return SFERR_NO_MEMORY;
	
	if ((error = pane->static_catch()) != SFERR_NO_ERROR) 
		return error;
	
	if ((error = pane->Append("Publisher", _itemAttributes->publisher)) != SFERR_NO_ERROR)
		return error;
			
	if ((error = pane->Append("Publish date", (_itemAttributes->publicationDate.AsUInt32() == 0) ? ("Unknown") : (_itemAttributes->publicationDate.Format("YYYY/MM/DD")))) != SFERR_NO_ERROR) 
		return error;
			
	if ((error = pane->Append("Type", _itemAttributes->binding)) != SFERR_NO_ERROR)
		return error;
			
	if ((error = pane->Append("ISBN", _asin)) != SFERR_NO_ERROR)
		return error;
			
	if ((error = pane->Append("Pages", _itemAttributes->numberOfPages)) != SFERR_NO_ERROR) 
		return error;

	error = pane->MakeLabels();
    return error;
}

//++++++++++++++++++++++End of AWSBook class implementation++++++++++++++++++++++++++//

//++++++++++++++++++++++++++AWSDvd class implementation+++++++++++++++++++++++++++++//
AWSDvd::AWSDvd(Void): AWSProduct()
{
	_itemAttributes = null;
	return;
}

AWSDvd::~AWSDvd(Void)
{
	if (_itemAttributes != null)
	{
		delete _itemAttributes;
		_itemAttributes = null;
	}

	return;
}

// Set ItemAtributes
Void	AWSDvd::SetItemAttributes(SFXAnsiString param)
{
	if (_flag == "Actor")
	{
		SFXAnsiStringPtr actor = new SFXAnsiString(DecodeHtml(param).GetCString());
		_itemAttributes->actors.Append(actor);
	}
	else if (_flag == "Director")
	{
		SFXAnsiStringPtr director = new SFXAnsiString(DecodeHtml(param).GetCString());
		_itemAttributes->directors.Append(director);
	}
	else if (_flag == "Format")
	{
		SFXAnsiStringPtr format = new SFXAnsiString(DecodeHtml(param).GetCString());
		_itemAttributes->formats.Append(format);
	}
	else if (_flag == "Amount")
		_itemAttributes->amount = param.AsSInt32();
	else if (_flag == "CurrencyCode")
		_itemAttributes->currencyCode = DecodeHtml(param);
	else if (_flag == "FormattedPrice")
		_itemAttributes->formattedPrice = DecodeHtml(param);
	else if (_flag == "Publisher")
		_itemAttributes->publisher = DecodeHtml(param);
	else if (_flag == "ReleaseDate")	
		_itemAttributes->releaseDate = StringToDate(param);
	else if (_flag == "RunningTime")
		_itemAttributes->runningTime = param.AsSInt32();
	else if (_flag == "Title")
		SetTitle(param);
}


//Create pane to display general information
SFCError AWSDvd::MakeGeneralPane(SFRTabControlPtr tab)
{
    SFCError error(SFERR_NO_ERROR);
    AWSTabPanePtr pane;
    UInt16 i;
    
	if (_itemAttributes == null) return SFERR_FAILED;
	
	if ((pane = ::new AWSTabPane(tab, "General")) == null) return SFERR_NO_MEMORY;

	if ((error = pane->static_catch()) != SFERR_NO_ERROR)
		return error;
    
	if ((error = pane->Append("Title", _title)) != SFERR_NO_ERROR)
		return error;

	if (_itemAttributes->directors.GetSize()>0)
	{
		SFXAnsiString directors = _itemAttributes->directors[0]->TrimLeft();
		for (i = 1; i < _itemAttributes->directors.GetSize(); ++i) 
			directors = directors + ", " + _itemAttributes->directors[i]->TrimLeft();
		if ((error = pane->Append("Director(s)", directors)) != SFERR_NO_ERROR)
			return error;
    }

	if ((error = pane->Append("Price", _itemAttributes->formattedPrice)) != SFERR_NO_ERROR)
		return error;
	 
	error = pane->MakeLabels();
    return error;
}

// create pane for detailed information
SFCError AWSDvd::MakeDetailPane(SFRTabControlPtr tab)
{
    SFCError error(SFERR_NO_ERROR);
    AWSTabPanePtr pane;
	UInt16 i;

	if (_itemAttributes == null) return SFERR_FAILED;
    
	if ((pane = ::new AWSTabPane(tab, "Details")) == null) return SFERR_NO_MEMORY;

	if ((error = pane->static_catch()) != SFERR_NO_ERROR) 
		return error;

	if (_itemAttributes->actors.GetSize()>0)
	{
		SFXAnsiString actors = _itemAttributes->actors[0]->TrimLeft();
		for (i = 1; i < _itemAttributes->actors.GetSize(); ++i) 
			actors = actors + ", " + _itemAttributes->actors[i]->TrimLeft();
		if ((error = pane->Append("Actor(s)", actors)) != SFERR_NO_ERROR)
			return error;
	}

	if (_itemAttributes->formats.GetSize()>0)
	{
		SFXAnsiString formats = _itemAttributes->formats[0]->TrimLeft();
		for (i = 1; i < _itemAttributes->formats.GetSize(); ++i) 
			formats = formats + ", " + _itemAttributes->formats[i]->TrimLeft();
		if ((error = pane->Append("Format", formats)) != SFERR_NO_ERROR)
			return error;
	}

	if ((error = pane->Append("ASIN", _asin)) != SFERR_NO_ERROR)
		return error;
	
	if ((error = pane->Append("Publisher", _itemAttributes->publisher)) != SFERR_NO_ERROR)
		return error;
			
	if ((error = pane->Append("Release date", (_itemAttributes->releaseDate.AsUInt32() == 0) ? ("Unknown") : (_itemAttributes->releaseDate.Format("YYYY/MM/DD")))) != SFERR_NO_ERROR) 
		return error;
	
	if ((error = pane->Append("Time (min)", _itemAttributes->runningTime)) != SFERR_NO_ERROR) 
		return error;

	error = pane->MakeLabels();
    return error;
}
