//
//      SearchWindow.cpp
//
//      This source code was automatically
//      generated by SophiaFramework 4.1.1
//
#include "pself3.hpp"
#include "SearchWindow.hpp"
#include "ListProductsWindow.hpp"
#include "SGRSoftkeyWindow.hpp"
#include "SGRSoftkeyToolbox.hpp"


SearchPane::SearchPane(SFRTabControlPtr tab, SFXWideStringConstRef title, SFXWideStringConstRef searchindex)
    : SFRTabPane(tab, title), _searchindex(searchindex)
{
	// Get BrowseNodes tiles and values from xml file to populate the combobox
	GetBrowseNodes();

	// register softkey handler
	RegisterHandler(static_cast<SFCEventEnum>(SGRSoftkeyWindow::USEREVT_SOFTKEY), HANDLER_AFTER, HANDLER_FUNCTION(OnSoftkey));
	
	static_throw(RegisterHandler(SFEVT_KEY, HANDLER_AFTER, HANDLER_FUNCTION(OnKey)));
	if (!static_try())return;
	
	//Add search components
	AddComponents();
		
	SGRSoftkeyWindowPtr softkey;
	SGRSoftkeyWindow::Initialize(this);

	if ((softkey = SGRSoftkeyWindow::GetInstance()) == null) return;

	static_throw(softkey->Register(this, SGRSoftkeyWindow::SOFTKEY_1, "End"));
	if (!static_try())	return;
	
	static_throw(softkey->Register(this, SGRSoftkeyWindow::SOFTKEY_2, "Store"));
	if (!static_try())return;
	
	static_throw(softkey->Register(this, SGRSoftkeyWindow::SOFTKEY_3, "Select"));
	if (!static_try()) return;
	
	SetReference(softkey->Bind(this));
	
    return;
}

SearchPane::~SearchPane(Void)
{
	SGRSoftkeyWindow::Terminate(this);
	
	delete [] BrowseNodesTitle;
	BrowseNodesTitle = null;

	delete [] BrowseNodesValue;
	BrowseNodesValue = null;

	_searchindex.Clear();
	_errmsg.Clear();

	return;
}

// softkey handler
HANDLER_IMPLEMENT_VOIDEVENT(SearchPane, OnSoftkey, event)
{
    switch (event.GetP16()) {
        case SGRSoftkeyWindow::SOFTKEY_1:
			// terminate application
			SFRApplication::Terminate();
            break;
        case SGRSoftkeyWindow::SOFTKEY_2:
			// display list of favorite products
			SFXAnsiString filename;
			SFCError error;
			SearchPanePtr pane = static_cast<SearchPanePtr>(GetDirector()->GetFront());

			AWSListProductsPtr _products;

			_products = ::new AWSListProducts();
			if (_products == null) return;

			filename= DIR_NAME + pane->_searchindex + ".dat";
            
			error = _products->GetFavoriteProducts(SFXPath(filename), pane->_searchindex);
			if (error == SFERR_NO_ERROR) 
				::new ListProductsWindow(_products, static_cast<SFXAnsiStringConstRef>(pane->_searchindex), FAVORITEWINDOW);

			break;
    }
    return;
}

// Key handler
HANDLER_IMPLEMENT_BOOLEVENT(SearchPane, OnKey, event)
{
    Bool ret(false);
	SearchPanePtr tmp;
			SFRResponderPtr responder(null);
    switch (event.GetP16()) {
      
		case AVK_UP:
			if (GetFocus()) 
				responder = GetFocus()->GetPrevious(TYPE_WILDCARD, ATTRIBUTE_WILDCARD, STATUS_VISIBLE | STATUS_ENABLE, false);
            
            if (responder) 
			{
                FocusPrevious();
                ret = true;
            } 

            break;

        case AVK_DOWN:
            FocusDown();
			ret= true;
			break;
    	case AVK_LEFT:
			tmp= static_cast<SearchPanePtr>(GetLeft(TYPE_WILDCARD,ATTRIBUTE_SFRTABPANE));
			if (tmp) tmp->Select();
			ret = true;
			break;
		case AVK_RIGHT:
			tmp= static_cast<SearchPanePtr>(GetRight(TYPE_WILDCARD,ATTRIBUTE_SFRTABPANE));
			if (tmp) tmp->Select();
			ret = true;
			break;
		default:
			ret = false;
            break;
    }

	// change the label of the third soft key accoding to the selected component
	SGRSoftkeyWindowPtr softkey;
	if (event.GetP16() == AVK_LEFT || event.GetP16() == AVK_RIGHT) 
	{
		if ((softkey = SGRSoftkeyWindow::GetInstance()) != null) 
		{
			static_throw(softkey->Register(this, SGRSoftkeyWindow::SOFTKEY_3, "Select"));
			if (static_try()) 
			{
				SetReference(softkey->Bind(this));
			}
		}
	}

	else if (event.GetP16() == AVK_UP || event.GetP16() == AVK_DOWN) 
	{
		if ((softkey = SGRSoftkeyWindow::GetInstance()) != null) 
		{
			SFRResponderPtr responder;
			if ((responder = GetFocus()) != null) 
			{
				// if the combo box is selected
				if (responder->GetAttribute() == ATTRIBUTE_SFRCOMBOBOXCONTROL) 
					softkey->Register(this, SGRSoftkeyWindow::SOFTKEY_3, "Select");
				// if the edit box is selected
				else if (responder->GetAttribute() == ATTRIBUTE_SFREDITBOXCONTROL) 
					softkey->Register(this, SGRSoftkeyWindow::SOFTKEY_3, "Input");
				// if the button is selected
				else if (responder->GetAttribute() == ATTRIBUTE_SFRBUTTONCONTROL) 
					softkey->Register(this, SGRSoftkeyWindow::SOFTKEY_3, "Search");
			}

			SetReference(softkey->Bind(this));
		}
	}

    return ret;
}

HANDLER_IMPLEMENT_VOIDCONTROL(SearchPane, OnTextControl, result, control) 
{
	// release the target
    control->SetStatusTarget(false);
	
}

// Search button pressed handler
HANDLER_IMPLEMENT_VOIDCONTROL(SearchPane, OnButtonControl, result, control) 
{
	SFRMultiDialog::ParamRec param;
	SFXWideString utf16str;

	SFXAnsiString _browsenode;
	SFXAnsiString _string, _keyword;

	// get browsenode 
	SInt16 ind = _combobox->GetValue();
	_browsenode = BrowseNodesValue[ind];

	_string = _textcontrol->GetText().Trim(); // trim spaces
	_keyword = _string.TrimRight("\r").TrimRight("\n"); // trim cariage return and linefeed

	if(_keyword.IsEmptyCString() && _browsenode =="All")// if no brosenode is selected, keyword must be supplied
		_errorDialog.Open("Keyword Error", "Please input keyword", DIALOG_OK, this);
	else // connect to AWS to get information
		_networkDialog.Open(AWSProduct::GetUrlForKeyWordSearch(_searchindex, _browsenode, _keyword), true, OnNetworkEntry, this, "Confirm", "Connect to the Internet?", "Yes", "No" );
}


Void SearchPane::OnNetworkEntry(SGRNetworkDialogSet::NotifyEnum notify, SFCError error, VoidPtr reference)
{
    SearchPanePtr(reference)->OnNetwork(notify, error);
}

Void SearchPane::OnNetwork(SGRNetworkDialogSet::NotifyEnum notify, SFCError error)
{
    unused(error);

    switch(notify) {
        case SGRNetworkDialogSet::NOTIFY_CONNECT_END:
            error = ParseXML();
			if(error != SFERR_NO_ERROR){
				_errorDialog.Open(XML_ERROR_TITLE, GetErrMsg(), DIALOG_OK, this);
			}
            break;
		default:
			break;
    }	
	return;
}

//get browsenodes from XML file to populate the combo box
SFCError SearchPane::GetBrowseNodes(Void)
{
	SFCError error(SFERR_NO_ERROR);
	SInt32 i;
	SFXAnsiString label;

    SFXXMLDOMParser parser;
    parser.SetDoNamespaces(false);

    error = parser.Parse(BROWSENODE_FILE); // parse XML file
    
	if (error != SFERR_NO_ERROR)
		return error;

    SFXXMLDocumentPtr root = parser.GetDocument(); // get the root
    
	// if parsing failed
	if (root==null)
		return SFERR_FAILED;

    
	SFXXMLNodePtr _anode;

    // Get list of Items
    SFXList<SFXXMLNodePtr>* list = root->GetElementsByTagName(_searchindex);
	
	if (list == null || list->GetSize() == 0)
		return SFERR_FAILED;
    
	_anode = list->Get(0); 
	SFXXMLNodePtr child(_anode);
	child = child->GetFirstChild();
	
	if (SFXAnsiString(child->GetNodeName()) == "Size") 
		_size = child->GetText().AsSInt32();
	else _size = BN_ARRAY_SIZE;

	BrowseNodesTitle = new SFXWideString[_size];
	BrowseNodesValue = new SFXAnsiString[_size];
			
	//get browsenode title and value
	for (i=0; i<_size && child != null; i++)
	{
		child = child->GetNextSibling();
		SFXXMLElementPtr element = static_cast<SFXXMLElementPtr>(child);
				
		BrowseNodesTitle[i] = element->GetText();
		BrowseNodesValue[i] = element->GetAttributes()->Item(0)->GetText();

	}

return error;
}

// new tracemap for retriving ASIN and Title from xml file
TagTracerPtr SearchPane::NewTraceMap(Void)
{
	// create trace map for retriving ASIN and Title
	TagTracerPtr root = new TagTracer();
	TagTracerPtr ItemSearchResponse = new TagTracer();
	root->Trace("ItemSearchResponse", ItemSearchResponse);
	TagTracerPtr Items = new TagTracer();
	ItemSearchResponse->Trace("Items", Items);
	TagTracerPtr Item = new TagTracer();
	Items->Trace("Item", Item);
	TagTracerPtr ASIN = new TagTracer(&AWSProduct::SetAsin);
	Item->Trace("ASIN", ASIN);
	TagTracerPtr ItemAttributes = new TagTracer();
	Item->Trace("ItemAttributes", ItemAttributes);
	TagTracerPtr Title = new TagTracer(&AWSProduct::SetTitle);
	ItemAttributes->Trace("Title", Title);
	
	return root;
}

// create XML file from received data and parse it to get products information
SFCError SearchPane::ParseXML(Void)
{
	SFCError error(SFERR_NO_ERROR);

	SFXBuffer _xmldata;

	// create filename
	SFXFile file;
	SFXPath filename;
	SFXPath path;
	SFXPath parentPath;
	Bool b;

	SInt16 i;

	for (i = 0; i < FILE_RETRY_TIMES; ++i) {
		
		// Get unique filename
		parentPath.Set(DIR_NAME);
		SFXFile::GetUniquePath(parentPath, "data", EXTENSION_NAME, &path);
		filename.Set(path);

		// if file exists then break
		SFXFile::Exists(filename, &b);
		if (!b) {
			break;
		}
	}

	if (i < FILE_RETRY_TIMES) 
	{
		// get the return XML document
		_xmldata = _networkDialog.GetData();

		// clear the text
		_networkDialog.ClearData();

		// delete this file
		SFXFile::Remove(filename);

		// open file in readwrite mode
		error = file.OpenReadWrite(filename);
		if (error != SFERR_NO_ERROR) 
		{
			_errmsg = "Cannot open XML file!";
			return error;
		}

		SFXBinaryStreamWriter writer;

		// Get writing streamB
		// 1st argument: buffer size
		// 2nd argument: pointer to store result
		
		error = file.GetStreamWriter(_xmldata.GetSize(), &writer);
		if (error != SFERR_NO_ERROR) 
		{
			_errmsg = "GetStreamWriter failed!";
			return error;	
		}	

		// write the SFXAnsiString to output stream
		error = writer.Write(_xmldata);
		if (error != SFERR_NO_ERROR) 
		{
			_errmsg = "Cannot write data to the output stream!";
			return error;
		}

		// flush the written data
		error = writer.Flush();
		if (error != SFERR_NO_ERROR) 
		{
			_errmsg = "Flush failed!";
			return error;
		}
		
		// close file
		file.Close();
		
		// parse xml file using SAXParser
	    AWSSAXParser			_parser(_searchindex);
		SFXXMLSAXParser			saxparser;
		
		// make the trace map to get list of products that contains only ASIN and Title
		_parser.SetRoot(NewTraceMap());

		// parse the xml file
		saxparser.SetDefaultHandler(&_parser);
		error = saxparser.Parse(filename);
		if (error != SFERR_NO_ERROR) 
		{
			_errmsg = "Parsing failed!";
			return error;
		}

		//delete xml file
		SFXFile::Remove(filename);

		// display the list of products
		if (_parser.GetProducts()->GetSize()<=0)
		{
			_errmsg = "No item found!";
			return 1;
		}

		if ((::new ListProductsWindow(_parser.GetProducts(), _searchindex, SEARCHWINDOW)) == null)
		{
			error = SFERR_NO_MEMORY;
			_errmsg = "Not enough memory!";
		}
    }
    else 
	{
		error = SFERR_FAILED;
		_errmsg = "XML parsing failed!";
    }
    
    return error;
}

//add search components to search pane
SFCError SearchPane::AddComponents(Void)
{
	SFCError error(SFERR_NO_ERROR);

	SFXRectangle rect(GetContentWorld());
    SInt16Const height = SFXGraphics::GetFontHeight(AEE_FONT_NORMAL);
    SInt16Const top = height;
	SInt16Const fontWidth = SFXGraphics::MeasureText(AEE_FONT_NORMAL, "@");

    // Label
    if ((::new SFRLabelControl(this, SFXRectangle(MARGIN, top, SFXGraphics::MeasureText(AEE_FONT_NORMAL, "Search:"), height), "Search:"))==null)
	    return SFERR_NO_MEMORY;

	
	//Combo box
	_combobox = ::new SFRComboboxControl(this, SFXRectangle(MARGIN, top + height, rect.GetWidth() - MARGIN * 2, height+4), BrowseNodesTitle, (SInt16)_size);
	if (_combobox == null) return SFERR_NO_MEMORY;

    
	// text control
	_textcontrol = ::new SFREditboxControl(this, SFXRectangle(MARGIN, top + height * 3, rect.GetWidth() - MARGIN * 2, height), "");
	if (_textcontrol == null) return SFERR_NO_MEMORY;
	
    // set max size
	_textcontrol->SetMaxSize(255);

	//register textcontrol handler
	static_throw(_textcontrol->RegisterHandler(SREVT_CONTROL, HANDLER_BEFORE,HANDLER_FUNCTION(OnTextControl)));
	if (static_try())
		RegisterHandler(SFEVT_KEY, AVK_SELECT, HANDLER_BEFORE,HANDLER_FUNCTION(SelectHandler));
	  
	SFXAnsiStringConst buttonString("Go!");

	// Button
    _button = ::new SFRButtonControl(this, SFXRectangle(MARGIN, top + height * 5, SFXGraphics::MeasureText(AEE_FONT_NORMAL, buttonString) + MARGIN, height + 10), buttonString);
	if (_button == null) return SFERR_NO_MEMORY;

	// register button handler
	_button->RegisterHandler(SREVT_CONTROL, HANDLER_BEFORE,HANDLER_FUNCTION(OnButtonControl));
	
	return error;
}

////////////////////////////////////////////////////////////

// Constructor
SearchWindow::SearchWindow(Void) : SFRPlainWindow(SFRApplication::GetInstance(),
    SGRSoftkeyToolbox::GetWindowPosition()) static_throws
{
    if (static_try())
	{
		// Calculate tab control coordinates
		SFXRectangle temp;
		temp = GetVirtualWorld();

		// Create tab control
		_tab = ::new SFRTabControl(this, temp);
		_tab->RegisterHandler(SFEVT_KEY, AVK_SELECT, HANDLER_AFTER, HANDLER_FUNCTION(TabSelect));
		
		//Create search pane for each product
		SearchPanePtr pane1, pane2;

		// 1st arg: tab control, 2nd arg: pane label, 3rd arg: product searchindex
		pane1 = ::new SearchPane(_tab, "Book", BOOK_SEARCH);
		pane2 = ::new SearchPane(_tab, "DVD", DVD_SEARCH);
		// set focus to first pane
		pane1->Select();

	}

    return;
}

// Destructor
SearchWindow::~SearchWindow(Void)
{
    return;
}


HANDLER_IMPLEMENT_VOIDVOID(SearchWindow, TabSelect)
{
	SearchPanePtr pane;

	// Get the topmost pane
	pane = static_cast<SearchPanePtr>(_tab->GetFront());

	if (pane != null)
		pane->SetStatusFocus(true);

    return;
}

